// TTY-Grin
// Copyright (C) 2001-2003 Daniel Beer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <stdio.h>
#include "message.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

message_part::message_part(void) {
}

message_part::message_part(const message_part& m) :
  _receipts(m._receipts), _headers(m._headers), _children(m._children),
  _data(m._data) {
}

message_part::~message_part(void) {
}

message_part&
message_part::operator=(const message_part& m) {
  _receipts=m._receipts;
  _headers=m._headers;
  _children=m._children;
  _data=m._data;

  return *this;
}

message_part::message_part(istream& in, const char *term,
			   list<string> *_raw) {
  char buf[1024];

  { // Read and assemble headers
    list<pair<string, string> > head;
    
    in.getline(buf, sizeof(buf));
    while(!(in.eof()||in.fail())) {
      chomp(buf);
      if(_raw) _raw->push_back(buf);
      if(!buf[0]) break;
      
      if(buf[0]=='\t') {
	if(head.size()) {
	  head.back().second+=' ';
	  head.back().second+=(buf+1);
	}
      } else {
	char *x=strchr(buf, ':');
	if(x) {
	  *(x++)=0;
	  if(*x) x++;
	  head.push_back(pair<string, string>(buf, x));
	}
      }
      in.getline(buf, sizeof(buf));
    }

    // Package headers
    for(list<pair<string, string> >::const_iterator i=head.begin();
	i!=head.end();++i) {
      if(!strcasecmp(i->first.c_str(), "received"))
	_receipts.push_back(i->second);
      else if(!(strcasecmp(i->first.c_str(), "content-type")&&
		strcasecmp(i->first.c_str(), "content-disposition"))) {
	string subtype;
	string value;
	int state=0;
	
	for(const char *data=i->second.c_str();*data;data++)
	  switch(state) {
	  case 0: // Toplevel value
	    if(*data==';') {
	      state=1;
	      _headers[i->first]=value;
	      value="";
	    } else if(!isspace(*data)) value+=*data;
	    break;
	  case 1: // Subtype
	    if(*data=='=') {
	      state=2;
	    } else if(!isspace(*data)) subtype+=*data;
	    break;
	  case 2: // Subtype value (token)
	    if(*data==';') {
	      state=1;
	      subtype.insert(0, ".");
	      subtype.insert(0, i->first);
	      _headers[subtype]=value;
	      subtype="";
	      value="";
	    } else if(*data=='"') state=2;
	    else if(!isspace(*data)) value+=*data;
	    break;
	  case 3:
	    if(*data=='=') state=2;
	    else value+=*data;
	    break;
	  }
	switch(state) {
	case 0:
	  _headers[i->first]=value;
	  break;
	case 2:
	case 3:
	  subtype.insert(0, ".");
	  subtype.insert(0, i->first);
	  _headers[subtype]=value;
	  break;
	}
      } else _headers[i->first]=i->second;
    }
  }

  message_part_two(in, term, _raw);
}

void
message_part::message_part_two(istream& in, const char *term,
			       list<string> *_raw) {
  char buf[1024];

  // Determine content characteristics

  string boundary;
  enum {PLAIN, BASE64, QUOTED} encoding=PLAIN;
  bool multipart=false;

  {
    header_list::const_iterator i=_headers.find("content-type");
    if(i!=_headers.end()) {
      if(!strncasecmp(i->second.c_str(), "multipart/", 10))
	multipart=true;
      else if(!strcasecmp(i->second.c_str(), "message/rfc822")) {
	_children.push_back(message_part(in, term, _raw));
	return;
      }
    }

    i=_headers.find("content-transfer-encoding");
    if(i!=_headers.end()) {
      if(!strcasecmp(i->second.c_str(), "base64"))
	encoding=BASE64;
      else if(!strcasecmp(i->second.c_str(), "quoted-printable"))
	encoding=QUOTED;
    }
  }

  int termlen=-1;
  if(term) termlen=strlen(term);

  in.getline(buf, sizeof(buf));
  while(!(in.eof()||in.fail())) {
    chomp(buf);
    if(_raw) _raw->push_back(buf);
    if(termlen>=0&&buf[0]=='-'&&buf[1]=='-'&&
       !strncmp(buf+2, term, termlen)) {
      in.putback('\n');
      for(const char *x=buf+strlen(buf)-1;x>=buf;x--) in.putback(*x);
      if(_raw) _raw->pop_back();
      break;
    }
    if(multipart&&buf[0]=='-'&&buf[1]=='-') {
      if(boundary.empty()) boundary=buf+2;
      if(!buf[boundary.size()+2])
	_children.push_back(message_part(in, boundary.c_str(), _raw));
    } else switch(encoding) {
    case PLAIN:
      _data.insert(_data.end(), buf, buf+strlen(buf));
      _data.push_back('\n');
      break;
    case QUOTED:
      {
	int i;
	for(i=0;buf[i];i++) {
	  if(buf[i]=='='&&buf[i+1]&&buf[i+2]) {
	    _data.push_back(hexdigit(buf[i+1])*16+hexdigit(buf[i+2]));
	    i+=2;
	  } else _data.push_back(buf[i]);
	}
	if(!(i&&buf[i-1]=='=')) _data.push_back('\n');
      }
      break;
    case BASE64:
      {
	int len=0, end=strlen(buf);
	for(int i=0;i<end;i+=4) if(buf[i+1]) {
	  buf[i+1]=to64(buf[i+1]);
	  buf[len++]=(to64(buf[i])<<2)|((buf[i+1]&0x30)>>4);
	  if(buf[i+2]&&buf[i+2]!='=') {
	    buf[i+2]=to64(buf[i+2]);
	    buf[len++]=((buf[i+1]&0x0f)<<4)|((buf[i+2]&0x3c)>>2);
	    if(buf[i+3]&&buf[i+3]!='=')
	      buf[len++]=((buf[i+2]&0x03)<<6)|to64(buf[i+3]);
	  }
	}
	_data.insert(_data.end(), buf, buf+len);
      }
      break;
    }
    in.getline(buf, sizeof(buf));
  }

  if(multipart) _headers["Content-Type.boundary"]=boundary;
  if(encoding!=BASE64&&_data.size()&&_data[_data.size()-1]=='\n')
    _data.pop_back();
}

char
message_part::to64(char x) {
  if(isupper(x)) return x-'A';
  if(islower(x)) return x-'a'+26;
  if(isdigit(x)) return x-'0'+52;
  if(x=='+') return 62;
  return 63;
}

void
message_part::send(ostream& out, const char *lend) const {
  for(receipt_list::const_iterator i=_receipts.begin();
      i!=_receipts.end();++i)
    out<<"Received: "<<*i<<lend;

  header_list::const_iterator last=_headers.end();
  for(header_list::const_iterator i=_headers.begin();
      i!=_headers.end();++i) {
    if(last!=_headers.end()&&
       !strncasecmp(last->first.c_str(), i->first.c_str(),
		    last->first.size())&&
       i->first[last->first.size()]=='.') {
      out<<"; "<<i->first.c_str()+last->first.size()+1<<'=';
      const char *x=i->second.c_str();
      while(isalnum(*x)||*x=='-'||*x=='.') x++;
      if(*x) out<<'"';
      out<<i->second;
      if(*x) out<<'"';
    } else {
      if(last!=_headers.end()) out<<lend;
      out<<i->first<<": "<<i->second;
      last=i;
    }
  }
  if(last!=_headers.end()) out<<lend;
  out<<lend;

  header_list::const_iterator i;

  i=_headers.find("content-transfer-encoding");
  if(i!=_headers.end()&&!strcasecmp(i->second.c_str(), "base64")) {
    static char alpha64[]=
      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

    int col=0;
    for(unsigned int j=0;j<_data.size();j+=3) {
      int len=j+2<_data.size()?3:_data.size()-j;
      unsigned char a=_data[j];
      unsigned char b=len>1?_data[j+1]:0;
      unsigned char c=len>2?_data[j+2]:0;
      out<<alpha64[a>>2]<<alpha64[((a&0x03)<<4)|((b&0xf0)>>4)]<<
	(len>1?alpha64[((b&0x0f)<<2)|((c&0xc0)>>6)]:'=')<<
	(len>2?alpha64[c&0x3f]:'=');
      if(++col>=18) {
	col=0;
	out<<lend;
      }
    }
    if(col) out<<lend;
  } else if(i!=_headers.end()&&
	    !strcasecmp(i->second.c_str(), "quoted-printable")) {
    char lastchar=0;
    int col=0;
    for(content_t::const_iterator j=_data.begin();j!=_data.end();j++) {
      if(strchr("!\"#$@[\\]^`{|}~=", *j)||(unsigned)*j>=127||
	 (*j<32&&*j!='\n'&&*j!='\r')) {
	char buf[4];
	snprintf(buf, sizeof(buf), "=%02X", *j);
	out<<buf;
	col+=3;
      } else {
	if(*j=='\n') col=0;
	else col++;
	if(lastchar=='.'&&(*j=='\r'||*j=='\n')) out<<' ';
	lastchar=*j;
	out<<*j;
      }
      if(col>=72) {
	out<<"="<<lend;
	col=0;
      }
    }
  } else {
    char lastchar=0;
    for(content_t::const_iterator j=_data.begin();j!=_data.end();j++) {
      if(lastchar=='.'&&(*j=='\r'||*j=='\n')) out<<' ';
      lastchar=*j;
      out<<*j;
    }
  }

  i=_headers.find("content-type");
  if(i!=_headers.end()) {
    if(!strncasecmp(i->second.c_str(), "multipart/", 10)) {
      string boundary;
      i=_headers.find("content-type.boundary");
      if(i!=_headers.end()) boundary=i->second;
      for(child_list::const_iterator c=_children.begin();
	  c!=_children.end();c++) {
	out<<"--"<<boundary<<lend;
	c->send(out, lend);
	out<<lend;
      }
      out<<"--"<<boundary<<"--"<<lend;
    } else if(!strcasecmp(i->second.c_str(), "message/rfc822")&&
	      _children.size())
      _children.front().send(out, lend);
  }
  out<<lend;
}

void
message::send_original(ostream& out, const char *lend) const {
  if(_raw.empty()) send(out, lend);
  else for(raw_t::const_iterator i=_raw.begin();i!=_raw.end();++i)
    out<<*i<<lend;
}
