// -*-Mode: C++;-*-

// TTY-Grin
// Copyright (C) 2001-2003 Daniel Beer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _CTREE_H
#define _CTREE_H

#include "slang.h"
#include <list>

class ctree;

class ctree_node {
public:
  ctree_node(ctree&, ctree_node * =0, ctree_node * =0);
  ~ctree_node(void);

  void move(ctree_node * =0, ctree_node * =0);

  bool get_expanded(void) const { return expanded; }
  void set_expanded(bool e) { expanded=e; }

  const char *get_text(unsigned int c) const { return text[c].c_str(); }
  void set_text(unsigned int c, const char *t) { text[c]=t; }

  ctree *get_owner(void) { return owner; }
  const ctree *get_owner(void) const { return owner; }

  ctree_node *get_parent(void) { return parent; }
  ctree_node *get_next(void) { return next; }
  ctree_node *get_prev(void) { return prev; }
  ctree_node *get_first(void) { return first; }
  ctree_node *get_last(void) { return last; }

  const ctree_node *get_parent(void) const { return parent; }
  const ctree_node *get_next(void) const { return next; }
  const ctree_node *get_prev(void) const { return prev; }
  const ctree_node *get_first(void) const { return first; }
  const ctree_node *get_last(void) const { return last; }

  const ctree_node *up(bool=false) const;
  const ctree_node *down(bool=false) const;

  ctree_node *up(bool e=false) {
    return (ctree_node *)((const ctree_node *)this)->up(e);
  }

  ctree_node *down(bool e=false) {
    return (ctree_node *)((const ctree_node *)this)->down(e);
  }

  const void *get_data(void) const { return data; }
  void *get_data(void) { return data; }
  void set_data(void *d) { data=d; }

private:
  bool expanded;
  string *text;

  ctree *owner;
  ctree_node *parent, *next, *prev, *first, *last;
  void *data;
};

class ctree {
  friend ctree_node;

public:
  ctree(unsigned int);
  virtual ~ctree(void);

  unsigned int get_width(unsigned int c) const { return widths[c]; }
  void set_width(unsigned int c, unsigned int w) { widths[c]=w; }

  unsigned int get_columns(void) const { return columns; }

  ctree_node *get_first(void) { return first; }
  const ctree_node *get_first(void) const { return first; }
  ctree_node *get_last(void) { return last; }
  const ctree_node *get_last(void) const { return last; }

  ctree_node *get_cursor(void) { return cursor; }
  const ctree_node *get_cursor(void) const { return cursor; }
  void set_cursor(ctree_node *c) { cursor=c; }

  int get_text_color(void) const { return textc; }
  void set_text_color(int t) { textc=t; }
  int get_cursor_color(void) const { return cursorc; }
  void set_cursor_color(int c) { cursorc=c; }

  unsigned char get_cursor_char(void) const { return cursor_char; }
  void set_cursor_char(unsigned char c) { cursor_char=c; }

  void run(void);

protected:
  virtual void winch(void);
  virtual bool keypress(int);
  virtual void draw(int, int, int, int);

private:
  unsigned int columns;
  unsigned int *widths;
  int textc, cursorc;
  unsigned char cursor_char;
  ctree_node *first, *last, *cursor, *scroll;
  char lastsearch[1024];

  static void sigwinch(int);
  void set_all(ctree_node *, bool);

  bool command_up(const char *);
  bool command_down(const char *);
  bool command_page_up(const char *);
  bool command_page_down(const char *);
  bool command_jump_top(const char *);
  bool command_jump_end(const char *);
  bool command_redraw(const char *);
  bool command_quit(const char *);
  bool command_find(const char *);
  bool command_all(const char *);
  bool command_collapse(const char *);
  bool command_expand(const char *);
};

#endif
