// TTY-Grin
// Copyright (C) 2001-2003 Daniel Beer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sys/types.h>
#include <slang.h>

#ifdef HAVE_DCGETTEXT
#include <libintl.h>
#else
#include "libintl.h"
#endif

#include "message.h"
#include "message_descriptor.h"
#include "newsgroup.h"
#include "mailbox.h"
#include "boxlist.h"

#define SLC_MAIN        0
#define SLC_STATUS      1
#define SLC_HEADERS     2
#define SLC_DIVIDER     3
#define SLC_CURSOR      4
#define SLC_SELECTION   5
#define SLC_QUOTED      6
#define SLC_SIGNATURE   7
#define SLC_INFO        8

#define N_(string) (string)
#define _(string) gettext(string)

extern void *sigdata_winch;

enum read_type_t {TEXT, PASSWORD, FILENAME};

bool read_line(int, int, int, const char *, char *, size_t, read_type_t=TEXT);
bool yes_or_no(const char *);
void report_error(const char *);
void prepare_message(message_part&, message_part&);
void pgp_printf(char *, size_t, const char *, int, const char **);

void config_init(void);
const char *config_get(const char *);
const char *config_name_to_mime(const char *);
const char *config_mime_handler(const char *);
const char *config_mime_filter(const char *);
bool config_should_kill(const char *);

struct tty_grin_help_key_t {
  const char *domain, *command, *comment;
};

struct tty_grin_help_t {
  const char *title, *comment;
  tty_grin_help_key_t *keys;
};

void tty_grin_help(const tty_grin_help_t&);

mailbox *tty_grin_pickbox(void);
void tty_grin_message(message, message_descriptor * =0);
void tty_grin_compose(message&);
void tty_grin_newsgroup(newsgroup&);
void tty_grin_mailbox(mailbox&, const char *);
void tty_grin_boxlist(boxlist&, const char *);
void tty_grin_news(void);
void tty_grin_imap4(void);
void tty_grin_addresses(void);
void tty_grin_menu(void);
bool tty_grin_groupbrowse(const char *, char *, size_t);
bool tty_grin_pickaddr(char *, size_t);

void kbd_init(void);
int kbd_name_to_key(const char *);
void kbd_bind(const char *, int, const char *);
const char *kbd_translate(const char *, int);
string kbd_reverse_translate(const char *, const char *);
void kbd_dump(ostream&);
